// $Id:$
//  Nllh = function(dat, lrf, pre, dist=10) {
//  # Compute Y-hat, SSE, and Negative Log LikeliHood from
//  #    dat  :    dat object with slots x, y, z of length len
//  #    lrf  :    Levy random field w/slots x y z of length n
//  #    pre  :    Precision (inverse of variance) for observations
//  #    dist :    distance scale, in km
//  # Note: Does not reflect regression on covariates ("beta")
//  if(missing(dat)) stop("Llh: Need Data object dat.");
//  if(missing(lrf)) stop("Llh: Need Levy random field lrf.");
//  if(missing(pre)) stop("Llh: Need Precision pre.");
//  if(length(dat)==0) return(0);             # Okay, sample from prior
//  if(lrf@bdy@Geo) {                         # Geodetic (lat/lon) coords?
//    R   = 6378.137;                         # Earth's radius in km.
//    myf = function (x, y) { cos((x-y)*pi/180); };
//  # See lrf-Main for higher-precision version of Geo distance formula...
//    costh =
//      cos(dat@y*pi/180) %o% cos(lrf@y*pi/180) *
//      outer(dat@x, lrf@x, myf) +
//      sin(dat@y*pi/180) %o% sin(lrf@y*pi/180);
//    d2 = (R*acos(costh))^2;                 # Squared dist in km^2
//  } else {                                  # Easy for Cartesian coords:
//    d2 = (outer(dat@x, lrf@x, "-"))^2 + (outer(dat@y, lrf@y, "-"))^2;
//  }
//  YHat = c(exp(-0.5*d2/dist^2) %*% lrf@z);  # Model prediction at wells
//  SSE  = crossprod(YHat-dat@z);           # Sum of squared residuals
//  nllh = (SSE*pre - length(dat)*log(pre))/2;# Negative log likelihood
//  return(list(nllh=nllh, SSE=SSE, pred=YHat));
//  }

// Compiled C subroutine to calculate negative log likelihood for LRF paper
//
#include <stdlib.h>
#include <math.h>
#define _SSIZE_T_DEFINED  /* To avoid a collision between S.h and ming */
#include <S.h>

#define SQR(x) ((x)*(x))


void Nllhc(double *datY, double *datX, 
             double *datZ, long   *datLen,
             long   *datGeo, double *lrfX,
             double *lrfY,   double *lrfZ,
             long   *lrfLen, double *pre, 
             double *dist,   double *nllh,
             double *SSE,    double *YHat) {
    int i, j;
    double d2, x, y, costh;
    const double R = 6378.137,      // Earth's radius in km.
      deg2rad = 0.01745329;         // pi/180

    *SSE = 0.0;
    for(i=0; i< *datLen ; ++i) {
      x = datX[i];
      y = datY[i];
      YHat[i] = 0.0;
      for(j=0; j< *lrfLen; ++j) {
        if(! *datGeo) { d2 = SQR(x-lrfX[j]) + SQR(y-lrfY[j]);
        } else {                   // Spherical geometry
          costh = 
            cos(y*deg2rad) * cos(lrfY[j]*deg2rad) * cos((x-lrfX[j])*deg2rad) +
            sin(y*deg2rad) * sin(lrfY[j]*deg2rad);
          d2 = SQR(R * acos(costh));
        }
        YHat[i] += lrfZ[j] * exp(-0.5 * d2 / SQR(*dist) );
      }
      *SSE += SQR(log(datZ[i]/YHat[i]));
//    *SSE += SQR(datZ[i] - YHat[i]);
    }
    *nllh = (*SSE * *pre - *datLen * log(*pre))/2;
  }
          
