// Compiled C subroutine to calculate the jump height sums for each sample 
// from the MCMC

#include "LevyInfer.h"
#include <S.h>


extern "C"{

void getLrfSamp(
  char **DatFL, char **DatVL,               // The filenames for the MCMC output
  long *sampNum,
  double *xlocs, double *ylocs,
  double *heights, long *nAtoms )          // The LRF will be returned in xlocs, ylocs,
                                           // and heights
{

  LevyInfer li( (*DatFL), (*DatVL), 1.0 );  // The value of dist will not matter for this purpose.

  int natoms = (int) (*nAtoms);
  Array1D<double> xlocsArr(natoms),
    ylocsArr(natoms),
    heightsArr(natoms);
  li.getLrfSamp((int) (*sampNum), xlocsArr, ylocsArr, heightsArr);
  for( int i=0; i<natoms; i++ ){
    xlocs[i] = xlocsArr[i];
    ylocs[i] = ylocsArr[i];
    heights[i] = heightsArr[i];
  }

  return;
}

}
          


