// Compiled C subroutine to calculate the estimated mean surface at a grid of points

#include "LevyInfer.h"
#include <S.h>


extern "C"{

void meanSurf(
  char **DatFL, char **DatVL,               // The base filename of the MCMC output
  double *dist,                   // The scale of the kernels
  double *xlocs, double *ylocs,   // The x and y coordinates of the locations at which the mean surface is desired.
  double *z, long *nLocs,
  long *sampNum )          // The calculated mean surface values should be written to z.
                                  // nLocs is the length of the xlocs, ylocs, and z vectors
{

  LevyInfer li( (*DatFL), (*DatVL), (*dist) );
  int nLocs_ = (*nLocs);

  Array1D<double> xlocsArr (nLocs_, xlocs),
    ylocsArr (nLocs_, ylocs),
    zArr (nLocs_);

  zArr = li.meanVals( xlocsArr, ylocsArr, (*sampNum) );

  for( int i=0; i<nLocs_; i++ )
    z[i] = zArr[i];

  return;
}

}
          


